/*
 * A memory pool providing regions from a preallocated amount of memory.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <systypes/base.h>

#include <list>
#include <mutex>

#include <mem/memory.h>



/* A memory pool abstraction. */

class MemoryPreallocated : public Memory
{
protected:
    std::mutex _lock;
    std::list<Region *> _regions;

    Memory *_memory;
    offset_t _amount;

public:
    explicit MemoryPreallocated(Memory *memory, offset_t amount);

    explicit MemoryPreallocated();

    virtual ~MemoryPreallocated();

    virtual Region *region();

    virtual offset_t region_size()
    { return _memory->region_size(); }

    virtual void release(Region *region);
};

// vim: tabstop=4 expandtab shiftwidth=4
